local addonName, L = ...;

local FLAMEBUFFICD = 0.575; -- approx. milliseconds (determined by experiment rather than calculation)

-- default settings
local defaults = {
					soundfile = "Sound\\Doodad\\SimonGame_LargeGreenTree.wav",
					buffname = "Burning Infusion",
					heatwarn = 90,
					idlewarn = 10,
					heatsize = 16,
					lock = false,
					sounds = true,
                    debugmode = false,
                 };
local lastgain,lastcount = 0,0;
local overheat = false;
local idle = true;
local enabled = false;

-- not local, called back by the timer function
function ICDExpired()
	if (FAVars.debugmode) then print("FA: icd over"); end
	if (not overheat) then
		FAFrameReady:Show();
		FAFrameBeware:Hide();
	else
		FAFrameBeware:Show();
		FAFrameReady:Hide();
	end
	FAFrameCooling:Hide();
end

local ticker = nil;
local function StartTimer()
	if (ticker) then
		ticker:Cancel();
	end
	ticker = C_Timer.NewTicker(FLAMEBUFFICD, ICDExpired, 1);
end

local function BuffCheck(count)
	--HeatCheck();
	if FAVars.debugmode then print("buff check: count "..count.." last "..lastcount); end
	if (count ~= lastcount) then
		if (count > lastcount) then -- GAINED ONE!
			local timenow = GetTime();
			StartTimer();
			if (FAVars.sounds) then
				if (PlaySoundFile(FAVars.soundfile) == nil) then
					PlaySound(FAVars.soundfile); -- sound is an identifier not a path
				end
			end
			if (lastgain and (lastgain + FLAMEBUFFICD > timenow)) then
				print("FA: second buff gained after "..(timenow-lastgain));
			end
			lastgain = timenow;
			idle = false;
			FAFrameReady:Hide();
			FAFrameBeware:Hide();
			FAFrameCooling:Show();
		end
		if (FAVars.sounds and count == 30) then
			PlaySound("A_MTERRACE_SELIMEMPOWERED");
		end
		lastcount = count;
		FAFrameCounter:SetText(count);
	end
end

local function HeatCheck(heat)
	if FAVars.debugmode then print("heat check "..heat.." overheat "..(overheat and "on" or "off").." idle "..(idle and "on" or "off")); end
	if (not overheat and heat >= FAVars.heatwarn) then
		if (FAVars.debugmode) then print("flamethrower overheating"); end
		overheat = true;
		ICDExpired(); -- change bg colour to amber
		if (FAVars.sounds) then
			PlaySound("A_CA_Sartharion_FireSpecial03");
		end
	end
	overheat = heat > FAVars.heatwarn;
	if (not idle and heat <= FAVars.idlewarn and (GetTime()-lastgain>10.0 or FAVars.debugmode)) then
		if FAVars.debugmode then print("flamethrower idle"); end
		idle = true;
		ICDExpired(); -- change bg colour back to green
		if (FAVars.sounds) then
			PlaySound("A_CA_Sartharion_FireSpecial04");
		end
	end
	FAFrameHeat:SetText(heat);
end

local function SetupHeatCounter()
	if (FAVars.heatsize>0) then
		FAFrameHeat:SetTextHeight(FAVars.heatsize);
		FAFrameHeat:Show();
	else
		FAFrameHeat:Hide();
	end
end

local function handler(msg)
    local command, arg = msg:match("^(%S*)%s*(.-)$");
	if (command == "debug") then
		FAVars.debugmode = not FAVars.debugmode;
		print("debug is : |cFF00FF00" .. (FAVars.debugmode and "true" or "false"));
		FAFrame:SetShown(FAVars.debugmode);
    elseif (command == "gain") then
		BuffCheck(lastcount+1);
		return;
    elseif (command == "lose") then
		BuffCheck(lastcount-1);
		return;
	elseif (command =="stack") then
		BuffCheck(tonumber(arg));
		return;
	elseif (command == "heat") then
		HeatCheck(tonumber(arg));
		return;
	elseif (command == "reset") then
		HeatCheck(0);
		BuffCheck(0);
		return;
	elseif (command == "soundstring") then
		if (arg and strlen(arg)>0 and (PlaySoundFile(arg) or PlaySound(arg))) then --check that its valid
			FAVars.soundfile = arg;
		end
	elseif (command == "sounds") then
		FAVars.sounds = not FAVars.sounds;
	elseif (command == "debuff") then
		if (arg and strlen(arg)>0) then
			FAVars.buffname = arg;
		end
	elseif (command == "warn") then
		if (arg and strlen(arg)>0) then
			FAVars.heatwarn = tonumber(arg);
		end
	elseif (command == "idle") then
		if (arg and strlen(arg)>0) then
			FAVars.idlewarn = tonumber(arg);
		end
	elseif (command == "lock") then
		FAVars.lock = not FAVars.lock;
	elseif (command == "heatsize") then
		if (arg and strlen(arg)>0) then
			FAVars.heatsize = tonumber(arg);
		end
		SetupHeatCounter();
	elseif (command == "defaults") then
		FAVars = defaults;
	else
		print("|cFFFF0000Flame Assist Version: |cFFFFFF00"..GetAddOnMetadata(addonName,"Version"));
		print("|cFF00FF00Options: /fa soundstring <path|ID> : |rChange the sound used for gaining a stack (PlaySoundFile or PlaySound compatible)");
 		print("|cFF00FF00Options: /fa sounds : |rToggle playing of all sounds");
		print("|cFF00FF00Options: /fa debuff <name> : |rChange the name of the stacking debuff to track (localisation)");
		print("|cFF00FF00Options: /fa warn <value> : |rChange the heat level to switch to warning colour");
		print("|cFF00FF00Options: /fa idle <value> : |rChange the heat level to switch back to normal colour");
		print("|cFF00FF00Options: /fa heatsize <value> : |rChange the size of the heat text (0 for off)");
		print("|cFF00FF00Options: /fa lock : |rToggle locking the position and size to prevent accidental changes");
		print("|cFF00FF00Options: /fa defaults : |rReset all options to default");
        print("|cFFFFFF00Debug Options: /fa debug gain lose stack heat reset");
        print("Left click to drag, right click to resize, when out of combat");
        print("");
    end
	print("current sound string is : |cFF00FF00",FAVars.soundfile);
	print("current debuff name is : |cFF00FF00",FAVars.buffname);
	print("current warn is : |cFF00FF00"..FAVars.heatwarn);
	print("current idle is : |cFF00FF00"..FAVars.idlewarn);
	print("current heatsize is : |cFF00FF00"..FAVars.heatsize);
	print("sounds are : ",(FAVars.sounds and "|cFF00FF00on" or "|cFFFF0000off"));
	print("frame is : "..(FAVars.lock and "|cFFFF0000" or "|cFF00FF00un").."locked");
end

SLASH_FASLASH1, SLASH_FASLASH2 = '/fa', '/flameassist';
SlashCmdList["FASLASH"] = handler;

function FA_OnLoad(self)
-- make draggable
	self:RegisterForDrag("LeftButton");
	self:SetMinResize(32,32);
-- register for all events we want
    self:RegisterEvent("UNIT_AURA");
    self:RegisterEvent("UNIT_POWER");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("ADDON_LOADED");
end

function FA_OnEvent(self, event, ...)
	if (event == "UNIT_AURA") then
		local unitname = ...;
		--if FAVars.debugmode then print("aura change "..unitname); end
		if (unitname == "player") then
			local ft = select(6,UnitBuff("player","Flamethrower"));
			local pickedup = ft ~= nil;
			-- check state of flamethrower "buff" for changes
			if (enabled and not pickedup) then
				enabled = false;
				FAFrame:Hide();
			elseif (not enabled and pickedup) then
				enabled = true;
				overheat = false;
				FAFrame:Show();
				FAFrameCounter:SetText("0");
				FAFrameHeat:SetText("0");
				if FAVars.debugmode then print("flamethrower picked up"); end
			end
			-- doing brackenspore
			if (enabled) then
				local count = select(4,UnitDebuff("player",FAVars.buffname));
				if (count == nil and lastcount>0) then -- stack has decayed or died?
					count = 0;
					if FAVars.debugmode then print("stack dropped"); end
				end
				if (count ~= nil) then
					BuffCheck(count);
				end
			end
		end
	elseif (event == "UNIT_POWER") then
		local unitname,ptype = ...;
		if (unitname == "player") then
			--if FAVars.debugmode then print("unit power "..ptype); end
			if (ptype == "ALTERNATE") then
				HeatCheck(UnitPower("player",ALTERNATE_POWER_INDEX));
			end
		end
	elseif (event == "PLAYER_ENTERING_WORLD") then
		FAFrameCounter:SetTextHeight(self:GetHeight()*2/3); -- doesnt work in onload :/
		FAFrame:SetShown(FAVars.debugmode);
		SetupHeatCounter();
	elseif (event == "ADDON_LOADED") then
		local loadedname = ...;
		if (addonName == loadedname) then
-- init saved vars table
			if (not FAVars) then
				FAVars = defaults;
			end
-- upgrade vars to 1.01
			if (FAVars.lock == nil) then
				FAVars.lock = defaults.lock;
				FAVars.idlewarn = defaults.idlewarn;
				FAVars.heatsize = defaults.heatsize;
				FAVars.sounds = defaults.sounds;
			end
		end
	end
end

function FA_StartMoving()
	if (not FAVars.lock and not InCombatLockdown()) then
		FAFrame:StartMoving();
	end
end

function FA_StartSizing()
	if (not FAVars.lock and not InCombatLockdown()) then
		FAFrame:StartSizing();
	end
end
